<?php

namespace Drupal\config_sync;

use Drupal\config_provider\InMemoryStorage;
use Drupal\config_provider\Plugin\ConfigCollectorInterface;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Config\StorageComparer;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Extension\Extension;

/**
 * The ConfigSyncSnapshotter provides helper functions for taking snapshots of
 * extension-provided configuration.
 */
class ConfigSyncSnapshotter implements ConfigSyncSnapshotterInterface {

  /**
   * The app root for the current operation.
   *
   * @var string
   */
  protected $root;

  /**
   * The configuration collector.
   *
   * @var \Drupal\config_provider\Plugin\ConfigCollectorInterface
   */
  protected $configCollector;

  /**
   * The snapshot config storage for values from the extension storage.
   *
   * @var \Drupal\Core\Config\StorageInterface
   */
  protected $snapshotStorage;

  /**
   * The configuration manager.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * Constructs a ConfigSyncSnapshotter object.
   *
   * @param string $root
   *   The app root.
   * @param \Drupal\config_provider\Plugin\ConfigCollectorInterface $config_collector
   *   The config collector.
   * @param \Drupal\Core\Config\StorageInterface $snapshot_storage
   *   The snapshot storage for the items from the extension storage.
   * @param \Drupal\Core\Config\ConfigManagerInterface $config_manager
   *   The configuration manager.
   */
  public function __construct($root, ConfigCollectorInterface $config_collector, StorageInterface $snapshot_storage, ConfigManagerInterface $config_manager) {
    $this->root = $root;
    $this->configCollector = $config_collector;
    $this->snapshotStorage = $snapshot_storage;
    $this->configManager = $config_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function refreshExtensionSnapshot($type, array $names) {
    $extensions = [];
    foreach ($names as $name) {
      $pathname = $this->drupalGetFilename($type, $name);
      $extensions[$name] = new Extension($this->root, $type, $pathname);
    }

    /* @var \Drupal\config_provider\InMemoryStorage $installable_config */
    $installable_config = $this->configCollector->getInstallableConfig($extensions);

    // Set up a storage comparer.
    $storage_comparer = new StorageComparer(
      $installable_config,
      $this->snapshotStorage,
      $this->configManager
    );
    $storage_comparer->createChangelist();

    $changelist = $storage_comparer->getChangelist();

    // Only add new or changed items.
    foreach (['create', 'update'] as $changelist_type) {
      if (isset($changelist[$changelist_type])) {
        foreach ($changelist[$changelist_type] as $item_name) {
          if ($provided_data = $installable_config->read($item_name)) {
            // Snapshot the configuration item as provided by the extension.
            $this->snapshotStorage->write($item_name, $provided_data);
          }
        }
      }
    }

    // Snapshot collections as well.
    foreach ($installable_config->getAllCollectionNames() as $collection) {
      $changelist = $storage_comparer->getChangelist(NULL, $collection);
      $installable_config = $installable_config->createCollection($collection);
      $snapshot_storage = $this->snapshotStorage->createCollection($collection);

      // Only add new or changed items.
      foreach (['create', 'update'] as $changelist_type) {
        if (isset($changelist[$changelist_type])) {
          foreach ($changelist[$changelist_type] as $item_name) {
            if ($provided_data = $installable_config->read($item_name)) {
              // Snapshot the configuration item as provided by the extension.
              $snapshot_storage->write($item_name, $provided_data);
            }
          }
        }
      }

    }

  }

  /**
   * {@inheritdoc}
   */
  public function createSnapshot() {
    $this->configManager->createSnapshot($this->configCollector->getInstallableConfig(), $this->snapshotStorage);
  }

  /**
   * Wrapper for drupal_get_filename().
   *
   * @param $type
   *   The type of the item; one of 'core', 'profile', 'module', 'theme', or
   *   'theme_engine'.
   * @param $name
   *   The name of the item for which the path is requested. Ignored for
   *   $type 'core'.
   *
   * @return string
   *   The path to the requested item or an empty string if the item is not
   *   found.
   */
  protected function drupalGetFilename($type, $name) {
    return drupal_get_filename($type, $name);
  }

}
